"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Analyzer {
    constructor() {
        this.featureRegExp = new RegExp('^(feature)+[:]?', 'i');
        this.scenarioRegExp = new RegExp('^(scenario)+[:]?', 'i');
        this.stateRegExp = new RegExp('^(given)+[:]?', 'i');
        this.actionRegExp = new RegExp('^(when)+[:]?', 'i');
        this.outcomeRegExp = new RegExp('^(then)+[:]?', 'i');
        this.glueRegExp = new RegExp('^(and|but)+[:]?', 'i');
    }
    getGherkins(listRows) {
        const features = [];
        const scenarios = [];
        const states = [];
        const actions = [];
        const outcomes = [];
        const files = [];
        let rowAnalyzed;
        let currentReadRow = '';
        for (const listRow of listRows) {
            if (rowAnalyzed = this.getValidFeature(listRow)) {
                features.push(rowAnalyzed);
                currentReadRow = 'feature';
            }
            else if (rowAnalyzed = this.getValidScenario(listRow)) {
                scenarios.push(rowAnalyzed);
                currentReadRow = 'scenario';
            }
            else if (rowAnalyzed = this.getValidState(listRow, currentReadRow)) {
                states.push(rowAnalyzed);
                currentReadRow = 'state';
            }
            else if (rowAnalyzed = this.getValidAction(listRow, currentReadRow)) {
                actions.push(rowAnalyzed);
                currentReadRow = 'action';
            }
            else if (rowAnalyzed = this.getValidOutcome(listRow, currentReadRow)) {
                outcomes.push(rowAnalyzed);
                currentReadRow = 'outcome';
            }
            else {
                continue;
            }
            files.push(listRow);
        }
        return { files, features, scenarios, states, actions, outcomes };
    }
    getValidFeature(listRow) {
        const listRowToAnalyze = this.featureRegExp.exec(listRow);
        return listRowToAnalyze ? listRow.replace(listRowToAnalyze[0], '').trim() : null;
    }
    getValidScenario(listRow) {
        const listRowToAnalyze = this.scenarioRegExp.exec(listRow);
        return listRowToAnalyze ? listRow.replace(listRowToAnalyze[0], '').trim() : null;
    }
    getValidState(listRow, previousRow) {
        const listRowToAnalyze = previousRow === 'state' ? this.glueRegExp.exec(listRow) : this.stateRegExp.exec(listRow);
        return listRowToAnalyze ? listRow.replace(listRowToAnalyze[0], '').trim() : null;
    }
    getValidAction(listRow, previousRow) {
        const listRowToAnalyze = previousRow === 'action' ? this.glueRegExp.exec(listRow) : this.actionRegExp.exec(listRow);
        return listRowToAnalyze ? listRow.replace(listRowToAnalyze[0], '').trim() : null;
    }
    getValidOutcome(listRow, previousRow) {
        const listRowToAnalyze = previousRow === 'outcome' ? this.glueRegExp.exec(listRow) : this.outcomeRegExp.exec(listRow);
        return listRowToAnalyze ? listRow.replace(listRowToAnalyze[0], '').trim() : null;
    }
}
exports.Analyzer = Analyzer;
