"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const util = require("util");
class Reader {
    constructor() {
        this.foldersToExlcude = ['node_modules', 'dist', '.git'];
        this.extensionToRead = '.feature';
        this.readDirAsync = util.promisify(fs.readdir);
        this.statAsync = util.promisify(fs.stat);
        this.readFileAsync = util.promisify(fs.readFile);
    }
    readFeatureFilesFromFolder(folder, onReadFilesFromFolder) {
        return __awaiter(this, void 0, void 0, function* () {
            let results = [];
            try {
                const readFilesList = yield this.readDirAsync(folder);
                let totalFilesToRead = readFilesList.length;
                if (!totalFilesToRead) {
                    return onReadFilesFromFolder(null, results);
                }
                readFilesList
                    .map((baseNameReadFile) => __awaiter(this, void 0, void 0, function* () {
                    let readFile = baseNameReadFile;
                    readFile = path.resolve(folder, readFile);
                    const stat = yield this.statAsync(readFile);
                    if (stat && stat.isDirectory()) {
                        this.readFeatureFilesFromFolder(readFile, (readFilesFromFolderError, readData) => {
                            if (!this.foldersToExlcude.includes(baseNameReadFile)) {
                                results = results.concat(readData);
                            }
                            if (!--totalFilesToRead) {
                                onReadFilesFromFolder(null, results);
                            }
                        });
                    }
                    else {
                        if (readFile.indexOf(this.extensionToRead) >= 0) {
                            results.push(readFile);
                        }
                        if (!--totalFilesToRead) {
                            onReadFilesFromFolder(null, results);
                        }
                    }
                }));
            }
            catch (readFeatureFilesFromFolderError) {
                onReadFilesFromFolder(readFeatureFilesFromFolderError, []);
            }
        });
    }
    readContentFeatureFile(file) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.readFileAsync(file, 'utf8');
            }
            catch (readContentFeatureFileError) {
                return '';
            }
        });
    }
    getRowsFeatureFile(contentFile) {
        let rowsFile = contentFile.split('\n');
        rowsFile = rowsFile
            .map(row => row.trim())
            .filter(row => !(row === ''));
        return rowsFile;
    }
}
exports.Reader = Reader;
